@@ -6,13 +6,22 @@ use Carp qw(croak);
 
 require Exporter;
 our @ISA         = qw(Exporter);
-our @EXPORT_OK   = qw(parse_vcd list_sigs get_timescale get_endtime);
+our @EXPORT_OK   = qw(
+                        parse_vcd list_sigs get_timescale get_endtime
+                        get_date get_version get_dumps
+                        get_decl_comments get_sim_comments
+);
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 my $timescale;
 my $endtime;
+my $date;
+my $version;
+my @decl_comments;
+my @sim_comments;
+my %dumps;
 
 sub list_sigs {
     # Parse input VCD file into data structure,
@@ -66,90 +75,131 @@ sub parse_vcd {
 
     my $use_stdout = (exists $opt->{use_stdout}) ? 1 : 0;
 
+    open my $fh, '<', $file or croak("Error: Can not open VCD file $file: $!");
+
+    # Parse declaration section of VCD file
     my %data;
     my $mult;
-    my $num_sigs;
     my @hier;
-    my $time = 0;
-    open my $fh, '<', $file or croak("Error: Can not open VCD file $file: $!");
+    @decl_comments = ();
     while (<$fh>) {
-        s/ \s+ \z //x;  # Remove trailing whitespace
-        s/ ^ \s+  //x;  # Remove leading  whitespace
-
-        if (/ \$enddefinitions \b /x) {
-            $num_sigs = scalar keys %data;
-            unless ($num_sigs) {
-                if ($all_sigs) {
-                    croak("Error: No signals were found in the VCD file $file.",
-                          'Check the VCD file for proper $var syntax.');
-                }
-                else {
-                    croak("Error: No matching signals were found in the VCD file $file.",
-                          ' Use list_sigs to view all signals in the VCD file.');
-                }
-            }
-            if (($num_sigs>1) and $use_stdout) {
-                croak("Error: There are too many signals ($num_sigs) for output ",
-                      'to STDOUT.  Use list_sigs to select a single signal.');
-            }
-            last if $only_sigs;
-        }
-
-        elsif (/ \$timescale \b /x) {
-            my $statement = $_;
-            my $line      = $_;
+        if (s/ ^ \s* \$ (\w+) \s+ //x) {
+            my $keyword = $1;
+            my $line = $_;
+            my @lines;
+            push @lines, $_ if length $_;
             while ($line !~ / \$end \b /x) {
                 $line = <$fh>;
-                $line =~ s/ \s+ \z //x;  # Remove trailing whitespace
-                $line =~ s/ ^ \s+  //x;  # Remove leading  whitespace
-                $statement .= " $line";
+                push @lines, $line;
             }
-            $mult = calc_mult($statement, $opt);
-        }
+            for my $line (@lines) {
+                trim($line);
+                $line =~ s/ \s* \$end \b //x;
+            }
+            pop @lines unless length $lines[-1];
 
-        elsif (/ \$scope \b /x) {
-            # assumes all on one line
-            #   $scope module dff $end
-            push @hier, (split)[2]; # just keep scope name
-        }
-        elsif (/ \$upscope \b /x) {
-            pop @hier;
-        }
-        elsif (/ \$var \b /x) {
-            # assumes all on one line:
-            #   $var reg 1 *@ data $end
-            #   $var wire 4 ) addr [3:0] $end
-            my (undef, $type, $size, $code, $name) = split /\s+/, $_, 5;
-            $name =~ s/ \s+ \$end .* //x;
-            $name =~ s/ \s //xg;
-            my $path = join '.', @hier;
-            my $full_name = "$path.$name";
-            push @{ $data{$code}{nets} }, {
-                type => $type,
-                name => $name,
-                size => $size,
-                hier => $path,
-            } if exists $usigs{$full_name} or $all_sigs;
-        }
-
-        elsif (/ ^ [#] (\d+) /x) {
-            $time = $mult * $1;
-            $endtime = $time;
-        }
-
-        elsif (/ ^ ([01zx]) (.+) /xi  or / ^ [br] (\S+) \s+ (.+) /xi) {
-            my $value = lc $1;
-            my $code  = $2;
-            if (exists $data{$code}) {
-                if ($use_stdout) {
-                    print "$time $value\n";
-                }
-                else {
-                    push @{ $data{$code}{tv} }, [$time, $value];
-                }
+            if ($keyword eq 'date') {
+                $date = join "\n", @lines;
+            }
+            elsif ($keyword eq 'version') {
+                $version = join "\n", @lines;
+            }
+            elsif ($keyword eq 'comment') {
+                push @decl_comments, join "\n", @lines;
+            }
+            elsif ($keyword eq 'timescale') {
+                $mult = calc_mult("@lines", $opt);
+            }
+            elsif ($keyword eq 'scope') {
+                my $scope = "@lines";
+                push @hier, (split /\s+/, $scope)[1]; # just keep scope name
+            }
+            elsif ($keyword eq 'upscope') {
+                pop @hier;
+            }
+            elsif ($keyword eq 'var') {
+                my $var = "@lines";
+                #   $var reg 1 *@ data $end
+                #   $var wire 4 ) addr [3:0] $end
+                my ($type, $size, $code, $name) = split /\s+/, $var, 4;
+                $name =~ s/ \s //xg;
+                my $path = join '.', @hier;
+                my $full_name = "$path.$name";
+                push @{ $data{$code}{nets} }, {
+                    type => $type,
+                    name => $name,
+                    size => $size,
+                    hier => $path,
+                } if exists $usigs{$full_name} or $all_sigs;
+            }
+            else { # enddefinitions
+                last;
             }
         }
     }
+
+    my $num_sigs = scalar keys %data;
+    unless ($num_sigs) {
+        if ($all_sigs) {
+            croak("Error: No signals were found in the VCD file $file.",
+                  'Check the VCD file for proper $var syntax.');
+        }
+        else {
+            croak("Error: No matching signals were found in the VCD file $file.",
+                  ' Use list_sigs to view all signals in the VCD file.');
+        }
+    }
+    if (($num_sigs>1) and $use_stdout) {
+        croak("Error: There are too many signals ($num_sigs) for output ",
+              'to STDOUT.  Use list_sigs to select a single signal.');
+    }
+
+    unless ($only_sigs) {
+        # Parse simulation section of VCD file
+        # Continue reading file
+        @sim_comments = ();
+        %dumps = ();
+        my $time = 0;
+        while (<$fh>) {
+            trim($_);
+            if (s/ ^ \$ comment \s* //x) {
+                my $line = $_;
+                my @lines;
+                push @lines, $_ if length $_;
+                while ($line !~ / \$end \b /x) {
+                    $line = <$fh>;
+                    push @lines, $line;
+                }
+                for my $line (@lines) {
+                    trim($line);
+                    $line =~ s/ \s* \$end \b //x;
+                }
+                pop @lines unless length $lines[-1];
+                my $comment = join "\n", @lines;
+                push @sim_comments, {time => $time, comment => $comment};
+            }
+            elsif (/ ^ \$ (dump \w+) /x) {
+                push @{ $dumps{$1} }, $time;
+            }
+            elsif (/ ^ [#] (\d+) /x) {
+                $time = $mult * $1;
+            }
+            elsif (/ ^ ([01zx]) (.+) /xi  or / ^ [br] (\S+) \s+ (.+) /xi) {
+                my $value = lc $1;
+                my $code  = $2;
+                if (exists $data{$code}) {
+                    if ($use_stdout) {
+                        print "$time $value\n";
+                    }
+                    else {
+                        push @{ $data{$code}{tv} }, [$time, $value];
+                    }
+                }
+            }
+        }
+        $endtime = $time;
+    }
+
     close $fh;
 
     return \%data;
@@ -157,18 +207,10 @@ sub parse_vcd {
 
 sub calc_mult {
     # Calculate a new multiplier for time values.
-    # Input statement is complete timescale, for example:
-    #   $timescale 10ns $end
-    # Input new_units is one of s|ms|us|ns|ps|fs.
     # Return numeric multiplier.
     # Also sets the package $timescale variable.
 
-    my ($statement, $opt) = @_;
-
-    my @fields = split /\s+/, $statement;
-    pop   @fields;  # delete $end from array
-    shift @fields;  # delete $timescale from array
-    my $tscale = join '', @fields;
+    my ($tscale, $opt) = @_;
 
     my $new_units;
     if (exists $opt->{timescale}) {
@@ -183,7 +225,7 @@ sub calc_mult {
 
     my $mult;
     my $units;
-    if ($tscale =~ / (\d+) ([a-z]+) /xi) {
+    if ($tscale =~ / (\d+) \s* ([a-z]+) /xi) {
         $mult  = $1;
         $units = lc $2;
     }
@@ -223,6 +265,12 @@ sub calc_mult {
     return (($mult * $scale) / $new_scale);
 }
 
+sub trim {
+    # Modify input string in-place
+    $_[0] =~ s/ \s+ \z //x;  # Remove trailing whitespace
+    $_[0] =~ s/ ^ \s+  //x;  # Remove leading  whitespace
+}
+
 sub get_timescale {
     return $timescale;
 }
@@ -231,6 +279,26 @@ sub get_endtime {
     return $endtime;
 }
 
+sub get_date {
+    return $date;
+}
+
+sub get_version {
+    return $version;
+}
+
+sub get_decl_comments {
+    return @decl_comments;
+}
+
+sub get_sim_comments {
+    return @sim_comments;
+}
+
+sub get_dumps {
+    return %dumps;
+}
+
 
 =head1 NAME
 
@@ -238,7 +306,7 @@ Verilog::VCD - Parse a Verilog VCD text file
 
 =head1 VERSION
 
-This document refers to Verilog::VCD version 0.05.
+This document refers to Verilog::VCD version 0.06.
 
 =head1 SYNOPSIS
 
@@ -419,6 +487,68 @@ If called before a file is parsed, it returns an undefined value.
     my $vcd = parse_vcd($file); # Parse a file first
     my $et  = get_endtime();    # Then query the endtime
 
+=head2 get_date( )
+
+This returns a string corresponding to the date as specified
+by the C<$date> VCD keyword.  It returns the date for
+the last VCD file parsed.  If called before a file is parsed, it
+returns an undefined value.
+
+    my $vcd  = parse_vcd($file); # Parse a file first
+    my $date = get_date();       # Then query the date
+
+=head2 get_version( )
+
+This returns a string corresponding to the version as specified
+by the C<$version> VCD keyword.  It returns the version for
+the last VCD file parsed.  If called before a file is parsed, it
+returns an undefined value.
+
+    my $vcd     = parse_vcd($file); # Parse a file first
+    my $version = get_version();    # Then query the version
+
+=head2 get_decl_comments( )
+
+This returns an array corresponding to the comments as specified by the
+C<$comment> VCD keyword in the declaration section of the VCD file.  There
+may be any number of C<$comment> keywords in the file.  It returns the
+comments for the last VCD file parsed.  If called before a file is parsed,
+it returns an empty array.  If there are no C<$comment> keywords in the
+declaration section of the file, it returns an empty array.
+
+    my $vcd   = parse_vcd($file);    # Parse a file first
+    my @comms = get_decl_comments(); # Then query the comments
+
+=head2 get_sim_comments( )
+
+This returns an array-of-hashes corresponding to the comments as specified
+by the C<$comment> VCD keyword in the simulation section of the VCD file.
+There may be any number of C<$comment> keywords in the file.  It returns
+the comments for the last VCD file parsed.  If called before a file is
+parsed, it returns an empty array.  If there are no C<$comment> keywords in
+the declaration section of the file, it returns an empty array.
+
+    my $vcd   = parse_vcd($file);    # Parse a file first
+    my @comms = get_sim_comments();  # Then query the comments
+
+The time at which the comment occurred is included with each comment.
+An example returned structure is:
+
+    ({time => 123, comment => '1st comment'},
+     {time => 456, comment => '2nd comment'})
+
+=head2 get_dumps( )
+
+This returns a hash-of-arrays corresponding to the C<$dump*> VCD keywords
+(C<$dumpvars>, C<$dumpon>, etc.).  in the declaration section of the VCD
+file.  The keys of the hash are the keywords, and the values are the
+simulation times at which the keywords occurred.  It returns the dump
+commands for the last VCD file parsed.  If called before a file is parsed,
+it returns an empty hash.
+
+    my $vcd   = parse_vcd($file);   # Parse a file first
+    my %dumps = get_dumps();        # Then query the dump commands
+
 =head1 EXPORT
 
 Nothing is exported by default.  Functions may be exported individually, or
@@ -431,12 +561,6 @@ L<Carp|Carp> Core module.
 
 =head1 LIMITATIONS
 
-Only the following VCD keywords are parsed:
-
-    $end                $scope
-    $enddefinitions     $upscope
-    $timescale          $var
-
 The extended VCD format (with strength information) is not supported.
 
 The default mode of C<parse_vcd> is to load the entire VCD file into the
